@@ -3,7 +3,6 @@ Fidelity (or Faithfulness) metrics
 """
 
 from inspect import isfunction
-from math import floor
 
 import numpy as np
 import tensorflow as tf
@@ -11,7 +10,7 @@ from scipy.stats import spearmanr
 
 from .base import ExplanationMetric
 from ..commons import batch_predictions_one_hot
-from ..types import Union, Callable, Optional
+from ..types import Union, Callable, Optional, Dict
 
 
 class MuFidelity(ExplanationMetric):
@@ -76,8 +75,9 @@ class MuFidelity(ExplanationMetric):
 
         # prepare the random masks that will designate the modified subset (S in original equation)
         # we ensure the masks have exactly `subset_size` pixels set to baseline
-        subset_masks = np.random.rand(self.nb_samples, self.grid_size ** 2).argsort(axis=-1) > \
-                       self.subset_size
+        subset_masks = np.random.rand(self.nb_samples, self.grid_size ** 2)
+        subset_masks = subset_masks.argsort(axis=-1) > self.subset_size
+
         # and interpolate them if needed
         subset_masks = subset_masks.astype(np.float32).reshape(
             (self.nb_samples, self.grid_size, self.grid_size, 1))
@@ -105,7 +105,8 @@ class MuFidelity(ExplanationMetric):
         """
         explanations = np.array(explanations)
         assert len(explanations) == len(self.inputs), "The number of explanations must be the " \
-                                                      "same as the number of inputs"
+                                            f"same as the number of inputs: {len(explanations)}"\
+                                            f" vs {len(self.inputs)}"
 
         correlations = []
         for inp, label, phi, base in zip(self.inputs, self.targets, explanations,
@@ -154,6 +155,7 @@ class CausalFidelity(ExplanationMetric):
         Value of the baseline state, will be called with the inputs if it is a function.
     steps
         Number of steps between the start and the end state.
+        Can be set to -1 for all possible steps to be computed.
     max_percentage_perturbed
         Maximum percentage of the input perturbed.
     """
@@ -167,18 +169,22 @@ class CausalFidelity(ExplanationMetric):
                  baseline_mode: Union[float, Callable] = 0.0,
                  steps: int = 10,
                  max_percentage_perturbed: float = 1.0,
-                 ): # pylint: disable=R0913
+                 ):
+        # pylint: disable=R0913
         super().__init__(model, inputs, targets, batch_size)
         self.causal_mode = causal_mode
         self.baseline_mode = baseline_mode
-        self.steps = steps
 
         self.nb_features = np.prod(inputs.shape[1:-1])
         self.inputs_flatten = inputs.reshape((len(inputs), self.nb_features, inputs.shape[-1]))
 
         assert 0.0 < max_percentage_perturbed <= 1.0, "`max_percentage_perturbed` must be" \
-                                                      "in ]O, 1]."
-        self.max_percentage_perturbed = max_percentage_perturbed
+                                                      "in ]0, 1]."
+        self.max_nb_perturbed = tf.math.floor(self.nb_features * max_percentage_perturbed)
+
+        if steps == -1:
+            steps = self.max_nb_perturbed
+        self.steps = steps
 
     def evaluate(self,
                  explanations: Union[tf.Tensor, np.ndarray]) -> float:
@@ -196,9 +202,46 @@ class CausalFidelity(ExplanationMetric):
             Metric score, area over the deletion (lower is better) or insertion (higher is
             better) curve.
         """
+        scores_dict = self.detailed_evaluate(explanations)
+
+        # compute auc using trapezoidal rule (the steps are equally distributed)
+        np_scores = np.array(list(scores_dict.values()))
+        auc = np.mean(np_scores[:-1] + np_scores[1:]) * 0.5
+
+        return auc
+
+    def detailed_evaluate(self,
+                          explanations: Union[tf.Tensor, np.ndarray]) -> Dict[int, float]:
+        """
+        Evaluate model performance for successive perturbations of an input.
+        Used to compute causal score.
+
+        The successive perturbations in the Insertion and Deletion metrics create a list of scores.
+        This list of scores make a score evolution curve.
+        The AUC of such curve is used as an explanation metric.
+        However, the curve in itself is rich in information,
+        its visualization and interpretation can bring further comprehension
+        on the explanation and the model.
+        Therefore this method was added so that it is possible to construct such curves.
+
+        Parameters
+        ----------
+        explanations
+            Explanation for the inputs, labels to evaluate.
+
+        Returns
+        -------
+        causal_score_dict
+            Dictionary of scores obtain for different perturbations
+            Keys are the steps, i.e the number of features perturbed
+            Values are the scores, the score of the model
+                on the inputs with the corresponding number of features perturbed
+        """
         explanations = np.array(explanations)
         assert len(explanations) == len(self.inputs), "The number of explanations must be the " \
-                                                      "same as the number of inputs"
+                                            f"same as the number of inputs: {len(explanations)}"\
+                                            f"vs {len(self.inputs)}"
+
         # the reference does not specify how to manage the channels of the explanations
         if len(explanations.shape) == 4:
             explanations = np.mean(explanations, -1)
@@ -212,10 +255,8 @@ class CausalFidelity(ExplanationMetric):
             np.ones_like(self.inputs, dtype=np.float32) * self.baseline_mode
         baselines_flatten = baselines.reshape(self.inputs_flatten.shape)
 
-        steps = np.linspace(0, self.nb_features * self.max_percentage_perturbed,self.steps,
-                            dtype=np.int32)
+        steps = np.linspace(0, self.max_nb_perturbed, self.steps+1, dtype=np.int32)
 
-        scores = []
         if self.causal_mode == "deletion":
             start = self.inputs_flatten
             end = baselines_flatten
@@ -225,6 +266,7 @@ class CausalFidelity(ExplanationMetric):
         else:
             raise NotImplementedError(f'Unknown causal mode `{self.causal_mode}`.')
 
+        scores_dict = {}
         for step in steps:
             ids_to_flip = most_important_features[:, :step]
             batch_inputs = start.copy()
@@ -236,13 +278,10 @@ class CausalFidelity(ExplanationMetric):
 
             predictions = batch_predictions_one_hot(self.model, batch_inputs,
                                                     self.targets, self.batch_size)
-            scores.append(predictions)
 
-        # compute auc using trapezoidal rule (the steps are equally reparted)
-        avg_scores = np.mean(scores, -1)
-        auc = np.mean(avg_scores[:-1] + avg_scores[1:]) * 0.5
+            scores_dict[step] = np.mean(predictions)
 
-        return auc
+        return scores_dict
 
 
 class Deletion(CausalFidelity):
@@ -268,6 +307,7 @@ class Deletion(CausalFidelity):
         Value of the baseline state, will be called with the inputs if it is a function.
     steps
         Number of steps between the start and the end state.
+        Can be set to -1 for all possible steps to be computed.
     max_percentage_perturbed
         Maximum percentage of the input perturbed.
     """
@@ -281,7 +321,8 @@ class Deletion(CausalFidelity):
                  steps: int = 10,
                  max_percentage_perturbed: float = 1.0
                  ):
-        super().__init__(model, inputs, targets, batch_size, "deletion", baseline_mode, steps)
+        super().__init__(model, inputs, targets, batch_size, "deletion",
+                         baseline_mode, steps, max_percentage_perturbed)
 
 
 class Insertion(CausalFidelity):
@@ -307,6 +348,7 @@ class Insertion(CausalFidelity):
         Value of the baseline state, will be called with the inputs if it is a function.
     steps
         Number of steps between the start and the end state.
+        Can be set to -1 for all possible steps to be computed.
     max_percentage_perturbed
         Maximum percentage of the input perturbed.
     """
@@ -320,21 +362,14 @@ class Insertion(CausalFidelity):
                  steps: int = 10,
                  max_percentage_perturbed: float = 1.0
                  ):
-        super().__init__(model, inputs, targets, batch_size, "insertion", baseline_mode, steps)
+        super().__init__(model, inputs, targets, batch_size, "insertion",
+                         baseline_mode, steps, max_percentage_perturbed)
 
 
 class CausalFidelityTS(ExplanationMetric):
     """
     Used to compute the insertion and deletion metrics for Time Series explanations.
 
-    Adaptation of the insertion and deletion principle based on the perturbations suggested by:
-        Schlegel et al. in 2019 in their paper: Towards a Rigorous Evaluation of XAI Methods on...
-    4 baseline mode are supported:
-        float values: set the baseline to a fixed values
-        "zero": set the baseline to zero
-        "inverse": set the baseline to the maximum for each feature minus the input value
-        "negative": set the baseline by taking the inverse of the input values
-
     Parameters
     ----------
     model
@@ -369,7 +404,7 @@ class CausalFidelityTS(ExplanationMetric):
                  metric: str = "loss",
                  batch_size: Optional[int] = 64,
                  causal_mode: str = "deletion",
-                 baseline_mode: Union[float, str] = 0.0,
+                 baseline_mode: Union[float, Callable] = 0.0,
                  steps: int = 10,
                  max_percentage_perturbed: float = 1.0,
                  ):  # pylint: disable=R0913
@@ -387,19 +422,14 @@ class CausalFidelityTS(ExplanationMetric):
 
         assert 0 < max_percentage_perturbed <= 1, \
             "max_percentage_perturbed should be between 0 and 1"
-        max_nb_perturbation = floor(max_percentage_perturbed*self.nb_features)
+        self.max_nb_perturbed = int(self.nb_features * max_percentage_perturbed)
 
         if steps == -1:
-            steps = max_nb_perturbation
-        assert 1 < steps <= max_nb_perturbation, \
-            f"The number of steps must be between 2 and {max_nb_perturbation} (or '-1')."
-        self.steps = int(steps)
-        self.max_nb_perturbation = max_nb_perturbation
-        self.model_baseline = self.model.evaluate(self.inputs, self.targets,
-                                                  verbose=0, return_dict=True)[self.metric]
+            steps = self.max_nb_perturbed
+        self.steps = steps
 
     def evaluate(self,
-                 explanations: Union[tf.Tensor, np.array]) -> float:
+                 explanations: Union[tf.Tensor, np.ndarray]) -> float:
         """
         Evaluate the causal score for time series explanations.
 
@@ -407,13 +437,46 @@ class CausalFidelityTS(ExplanationMetric):
         ----------
         explanations
             Explanation for the inputs, labels to evaluate.
+
         Returns
         -------
         causal_score
-            Metric score, area over the deletion (lower is better) or insertion (higher is
-            better) curve.
-            The area represent the mean score perturbation compared to model_baseline
-                (model score on non-perturbed inputs)
+            Metric score (for interpretation, see score interpretation in the documentation).
+        """
+        scores_dict = self.detailed_evaluate(explanations)
+
+        # compute auc with trapeze
+        np_scores = np.array(list(scores_dict.values()))
+        auc = np.mean(np_scores[:-1] + np_scores[1:]) * 0.5
+
+        return auc
+
+    def detailed_evaluate(self,
+                          explanations: Union[tf.Tensor, np.ndarray]) -> Dict[int, float]:
+        """
+        Evaluate model performance for successive perturbations of an input.
+        Used to compute causal score for time series explanations.
+
+        The successive perturbations in the Insertion and Deletion metrics create a list of scores.
+        This list of scores make a score evolution curve.
+        The AUC of such curve is used as an explanation metric.
+        However, the curve in itself is rich in information,
+        its visualization and interpretation can bring further comprehension
+        on the explanation and the model.
+        Therefore this method was added so that it is possible to construct such curves.
+
+        Parameters
+        ----------
+        explanations
+            Explanation for the inputs, labels to evaluate.
+
+        Returns
+        -------
+        causal_score_dict
+            Dictionary of scores obtain for different perturbations
+            Keys are the steps, i.e the number of features perturbed
+            Values are the scores, the score of the model
+                on the inputs with the corresponding number of features perturbed
         """
         explanations = np.array(explanations)
         assert explanations.shape == self.inputs.shape, "The number of explanations must be the " \
@@ -424,26 +487,12 @@ class CausalFidelityTS(ExplanationMetric):
         # for each sample, sort by most important features according to the explanation
         most_important_features = np.argsort(explanations_flatten, axis=-1)[:, ::-1]
 
-        if isinstance(self.baseline_mode, float):
-            baselines = np.full(self.inputs.shape, self.baseline_mode, dtype=np.float32)
-        elif self.baseline_mode == "zero":
-            baselines = np.zeros(self.inputs.shape)
-        elif self.baseline_mode == "inverse":
-            time_ax = 1
-            maximums = self.inputs.max(axis=time_ax)
-            maximums = np.expand_dims(maximums, axis=time_ax)
-            maximums = np.repeat(maximums, self.inputs.shape[time_ax], axis=time_ax)
-            baselines = maximums - self.inputs
-        elif self.baseline_mode == "negative":
-            baselines = -self.inputs
-        else:
-            raise NotImplementedError(f'Unknown perturbation type `{self.baseline_mode}`.')
-
+        baselines = self.baseline_mode(self.inputs) if isfunction(self.baseline_mode) else \
+            np.full(self.inputs.shape, self.baseline_mode, dtype=np.float32)
         baselines_flatten = baselines.reshape(self.inputs_flatten.shape)
 
-        steps = np.linspace(1, self.max_nb_perturbation, self.steps, dtype=np.int32)
+        steps = np.linspace(0, self.max_nb_perturbed, self.steps+1, dtype=np.int32)
 
-        scores = []
         if self.causal_mode == "deletion":
             start = self.inputs_flatten
             end = baselines_flatten
@@ -453,6 +502,7 @@ class CausalFidelityTS(ExplanationMetric):
         else:
             raise NotImplementedError(f'Unknown causal mode `{self.causal_mode}`.')
 
+        scores_dict = {}
         for step in steps:
             ids_to_flip = most_important_features[:, :step]
             perturbed_inputs = start.copy()
@@ -465,23 +515,22 @@ class CausalFidelityTS(ExplanationMetric):
             score = self.model.evaluate(perturbed_inputs, self.targets,
                                         self.batch_size, verbose=0,
                                         return_dict=True)
-            scores.append(score[self.metric])
+            scores_dict[step] = score[self.metric]
 
-        auc = abs(np.mean(scores) - self.model_baseline)
-
-        return auc
+        return scores_dict
 
 
 class DeletionTS(CausalFidelityTS):
     """
     Adaptation of the deletion metric for time series.
 
-    The deletion metric measures the drop in the probability of a class
-    as the input is gradually perturbed. The feature - time-steps pairs
-    are perturbed in order of importance given by the explanation.
-    A sharp drop, and thus a small area under the probability curve,
+    The deletion metric measures the drop in the probability of a class as the input is
+    gradually perturbed. The feature - time-steps pairs are perturbed in order of importance
+    given by the explanation. A sharp drop, and thus a small area under the probability curve,
     are indicative of a good explanation.
 
+    Ref. Schlegel et al., Towards a Rigorous Evaluation of XAI Methods (2019).
+
     Parameters
     ----------
     model
@@ -496,11 +545,14 @@ class DeletionTS(CausalFidelityTS):
     batch_size
         Number of samples to explain at once, if None compute all at once.
     baseline_mode
-        Value of the baseline state, associated perturbation for strings.
+        Value of the baseline state or the associated perturbation functions.
+        A float value will fix the baseline to that value, "zero" set the baseline to zero,
+        "inverse" set the baseline to the maximum for each feature minus the input value and
+        "negative" set the baseline by taking the inverse of input values.
     steps
         Number of steps between the start and the end state.
         Can be set to -1 for all possible steps to be computed.
-    max_nb_perturbation
+    max_percentage_perturbed
         Maximum percentage of the input perturbed.
     """
 
@@ -510,7 +562,7 @@ class DeletionTS(CausalFidelityTS):
                  targets: Optional[Union[tf.Tensor, np.ndarray]] = None,
                  metric: str = "loss",
                  batch_size: Optional[int] = 64,
-                 baseline_mode: Union[float, str] = 0.0,
+                 baseline_mode: Union[float, Callable] = 0.0,
                  steps: int = 10,
                  max_percentage_perturbed: float = 1.0,
                  ):  # pylint: disable=R0913
@@ -522,11 +574,56 @@ class InsertionTS(CausalFidelityTS):
     """
     Adaptation of the insertion metric for time series.
 
-    The insertion metric, on the other hand, captures the importance of
-    the feature - time-steps pairs in terms of their ability to synthesize
-    a time series and is measured by the rise in the probability of the
-    class of interest as feature - time-steps pairs are added according to
-    the generated importance map.
+    The insertion metric, captures the importance of the feature - time-steps pairs in terms of
+    their ability to synthesize a time series.
+
+    Ref. Schlegel et al., Towards a Rigorous Evaluation of XAI Methods (2019).
+
+    Parameters
+    ----------
+    model
+        Model used for computing metric.
+    inputs
+        Input samples under study.
+    targets
+        One-hot encoded labels or regression target (e.g {+1, -1}), one for each sample.
+    metric
+        The metric used to evaluate the model performance. One of the model metric keys when calling
+        the evaluate function (e.g 'loss', 'accuracy'...). Default to loss.
+    batch_size
+        Number of samples to explain at once, if None compute all at once.
+        Value of the baseline state or the associated perturbation functions.
+        A float value will fix the baseline to that value, "zero" set the baseline to zero,
+        "inverse" set the baseline to the maximum for each feature minus the input value and
+        "negative" set the baseline by taking the inverse of input values.
+    steps
+        Number of steps between the start and the end state.
+        Can be set to -1 for all possible steps to be computed.
+    max_percentage_perturbed
+        Maximum percentage of the input perturbed.
+    """
+
+    def __init__(self,
+                 model: tf.keras.Model,
+                 inputs: Union[tf.data.Dataset, tf.Tensor, np.ndarray],
+                 targets: Optional[Union[tf.Tensor, np.ndarray]] = None,
+                 metric: str = "loss",
+                 batch_size: Optional[int] = 64,
+                 baseline_mode: Union[float, Callable] = 0.0,
+                 steps: int = 10,
+                 max_percentage_perturbed: float = 1.0,
+                 ):  # pylint: disable=R0913
+        super().__init__(model, inputs, targets, metric, batch_size,
+                         "insertion", baseline_mode, steps, max_percentage_perturbed)
+
+
+class DeletionTab(CausalFidelityTS):
+    """
+    Adaptation of the deletion metric for tabular data.
+
+    Ref. Petsiuk & al., RISE: Randomized Input Sampling for Explanation of Black-box Models (2018).
+    https://arxiv.org/pdf/1806.07421.pdf
+    Ref. Schlegel et al., Towards a Rigorous Evaluation of XAI Methods (2019).
 
     Parameters
     ----------
@@ -542,13 +639,13 @@ class InsertionTS(CausalFidelityTS):
     batch_size
         Number of samples to explain at once, if None compute all at once.
     baseline_mode
-        Value of the baseline state, associated perturbation for strings.
+        Value of the baseline state, will be called with the inputs if it is a function.
     steps
         Number of steps between the start and the end state.
         Can be set to -1 for all possible steps to be computed.
-    max_nb_perturbation
+    max_percentage_perturbed
         Maximum percentage of the input perturbed.
-    """
+    """ # pylint: disable=R0913
 
     def __init__(self,
                  model: tf.keras.Model,
@@ -556,9 +653,53 @@ class InsertionTS(CausalFidelityTS):
                  targets: Optional[Union[tf.Tensor, np.ndarray]] = None,
                  metric: str = "loss",
                  batch_size: Optional[int] = 64,
-                 baseline_mode: Union[float, str] = 0.0,
+                 baseline_mode: Union[float, Callable] = 0.0,
                  steps: int = 10,
                  max_percentage_perturbed: float = 1.0,
-                 ):  # pylint: disable=R0913
+                 ):
+        super().__init__(model, inputs, targets, metric, batch_size,
+                         "deletion", baseline_mode, steps, max_percentage_perturbed)
+
+
+class InsertionTab(CausalFidelityTS):
+    """
+    Adaptation of the insertion metric for tabular data.
+
+    Ref. Petsiuk & al., RISE: Randomized Input Sampling for Explanation of Black-box Models (2018).
+    https://arxiv.org/pdf/1806.07421.pdf
+    Ref. Schlegel et al., Towards a Rigorous Evaluation of XAI Methods (2019).
+
+    Parameters
+    ----------
+    model
+        Model used for computing metric.
+    inputs
+        Input samples under study.
+    targets
+        One-hot encoded labels or regression target (e.g {+1, -1}), one for each sample.
+    metric
+        The metric used to evaluate the model performance. One of the model metric keys when calling
+        the evaluate function (e.g 'loss', 'accuracy'...). Default to loss.
+    batch_size
+        Number of samples to explain at once, if None compute all at once.
+    baseline_mode
+        Value of the baseline state, will be called with the inputs if it is a function.
+    steps
+        Number of steps between the start and the end state.
+        Can be set to -1 for all possible steps to be computed.
+    max_percentage_perturbed
+        Maximum percentage of the input perturbed.
+    """ # pylint: disable=R0913
+
+    def __init__(self,
+                 model: tf.keras.Model,
+                 inputs: Union[tf.data.Dataset, tf.Tensor, np.ndarray],
+                 targets: Optional[Union[tf.Tensor, np.ndarray]] = None,
+                 metric: str = "loss",
+                 batch_size: Optional[int] = 64,
+                 baseline_mode: Union[float, Callable] = 0.0,
+                 steps: int = 10,
+                 max_percentage_perturbed: float = 1.0,
+                 ):
         super().__init__(model, inputs, targets, metric, batch_size,
                          "insertion", baseline_mode, steps, max_percentage_perturbed)
